# This Python file uses the following encoding: iso-8859-1
# Barnes and Noble book scraper
#
# Copyright  1999-2011 Readerware Corporation.  All Rights Reserved.

import  os
import  string

import  rw

from    com.readerware.http import HTTPConnection

from    rwsupport import searchFor
from    rwsupport import searchForPlus
from    rwsupport import stripText
from    rwsupport import appendComment

from    jarray import zeros, array
from    java.lang import String
from    java.util import Date


def extract(source):
    # Defaults
    rw.setDefaultPublicationPlace("US")
    rw.setDefaultLanguage("English")
    rw.setCurrencySymbol("$")


    # Used pricing
    i = string.find(source, "<em>Marketplace</em>")

    if i != -1:
        prices = searchForPlus(source, "<em>Marketplace</em>")
        prices = searchForPlus(prices, "class=\"price\"")
        prices = searchForPlus(prices, ">")
        i = string.find(prices, "<")
        rw.setUsedPrice(stripText(prices[0:i]));
        rw.setNewPrice(stripText(prices[0:i]));

        i = string.find(prices, "<em>Used</em>")

        if i != -1:
            counts = searchForPlus(prices, "<em>Used</em>")
            counts = searchForPlus(counts, "(")
            i = string.find(counts, ")")
            rw.setUsedCount(stripText(counts[0:i]));

        i = string.find(prices, ">New<")

        if i != -1:
            counts = searchForPlus(prices, ">New<")
            counts = searchForPlus(counts, "(")
            i = string.find(counts, ")")
            rw.setNewCount(stripText(counts[0:i]));

        


    # Main extraction
    # Find Image
    i = string.find(source, "id=\"prodImage\"")

    if i != -1:
        images = searchForPlus(source, "id=\"prodImage\"")
        image = searchForPlus(images, "src=\"")
        i = string.find(image, "\"")
        image = stripText(image[0:i])

        i = string.find(image, "0000000000")

        if i == -1:
            rw.setImage1("https:" + image)
            image = string.replace(image, "550x406", "600x595")
            rw.setLargeImage1("https:" + image)



    # Find Title
    source = searchForPlus(source, "id=\"prodSummary")
    source = searchForPlus(source, "<h1")
    source = searchForPlus(source, ">")
    i = string.find(source, "<")
    title = stripText(source[0:i])

    i = string.rfind(title, "(")

    if i != -1:
        title = stripText(title[0:i])

    rw.setTitle(title)


    # Find Author
    authorList = []
    i = string.find(source, "class=\"contributors")

    if i != -1:
        source = searchForPlus(source, "class=\"contributors")
        i = string.find(source, "</span>")
        authors = stripText(source[0:i+6])

        while (searchFor(authors, "href=\"") != None):
            authors = searchForPlus(authors, "href=\"")
            i = string.find(authors, "itemprop=\"author\"")

            if i != -1:
                authors = searchForPlus(authors, "itemprop=\"author\"")

            authors = searchForPlus(authors, "\">")
            i = string.find(authors, "<")
            author = stripText(authors[0:i])
            authors = searchForPlus(authors, "</")
            authors = stripText(authors)
            i = string.rfind(author, "(")

            if i != -1:
                role = stripText(author[i:])
                author = stripText(author[0:i])
                i = string.find(role, ")")
                role = stripText(role[0:i+1])

                if role == "(Illustrator)":
                    rw.setIllustrator(author)
                    author = ""
                elif role == "(Translator)":
                    rw.setTranslator(author)
                    author = ""
                elif role == "(Editor)":
                    rw.setEditor(author)
                    author = ""
                else:
                    author = author + " " + role
                
            if author != "":
                authorList.append(author)

    rw.setAuthors(authorList)


    # Find Format
    i = string.find(source, "itemprop=\"bookFormat\"")

    if i != -1:
        format = searchForPlus(source, "itemprop=\"bookFormat\"")
        format = searchForPlus(format, ">")
        i = string.find(format, "<")
        format = stripText(format[0:i])
        rw.setFormat(format)


    # Find Rating
    i = string.find(source, "class=\"starDisplay\"")

    if i != -1:
        rating = searchForPlus(source, "class=\"starDisplay\"")
        rating = searchForPlus(rating, "class=\"stars-")
        rating = searchForPlus(rating, " r")
        i = string.find(rating, "\"")
        rating = stripText(rating[0:i])

        if rating.endswith("h") == 1:
            rating = rating[0:len(rating)-1] + " 1/2"

        rw.setMyRating(rating)


    # ISBN
    i = string.find(source, ">ISBN-13:")

    if i != -1:
        isbn = searchForPlus(source, ">ISBN-13:")
        isbn = searchForPlus(isbn, "<td>")
        i = string.find(isbn, "<")
        rw.setISBN(stripText(isbn[0:i]))


    # Find Publisher
    i = string.find(source, ">Publisher:")

    if i != -1:
        publisher = searchForPlus(source, ">Publisher:")
        publisher = searchForPlus(publisher, "<td>")
        i = string.find(publisher, "</td>")
        publisher = stripText(publisher[0:i])

        if publisher.startswith("<a href") == 1:
            publisher = searchForPlus(publisher, "\">")
            i = string.find(publisher, "itemprop=\"publisher\">")

            if i != -1:
                publisher = searchForPlus(publisher, "itemprop=\"publisher\">")

            i = string.find(publisher, "<")
            publisher = stripText(publisher[0:i])

        rw.setPublisher(publisher)


    # Find Publication date
    i = string.find(source, ">Publication date:")

    if i != -1:
        date = searchForPlus(source, ">Publication date:")
        date = searchForPlus(date, "<td>")
        i = string.find(date, "<")
        rw.setReleaseDate(stripText(date[0:i]))


    # Find Pages
    i = string.find(source, ">Pages:")

    if i != -1:
        pages = searchForPlus(source, ">Pages:")
        pages = searchForPlus(pages, "<td>")
        i = string.find(pages, "<")
        rw.setPages(stripText(pages[0:i]))


    # Find Sales rank
    i = string.find(source, ">Sales rank:")

    if i != -1:
        rank = searchForPlus(source, ">Sales rank:")
        rank = searchForPlus(rank, "<td>")
        i = string.find(rank, "<")
        rw.setSalesRank(stripText(rank[0:i]))


    # Series
    i = string.find(source, ">Series:<")

    if i != -1:
        series = searchForPlus(source, ">Series:<")
        series = searchForPlus(series, "href=\"")
        series = searchForPlus(series, ">")
        i = string.find(series, "<")
        rw.setSeries(stripText(series[0:i]))

        number = searchForPlus(source, ">Series:<")
        number = searchForPlus(number, "\">")
        i = string.find(number, "</td>")
        number = stripText(number[0:i])
        i = string.find(number, "#")

        if i != -1:
            number = searchForPlus(number, "#")
            number = stripText(number)
            rw.setSeriesNumber(number)


    # Dimensions
    i = string.find(source, ">Product dimensions:")

    if i != -1:
        dimensions = searchForPlus(source, ">Product dimensions:")
        dimensions = searchForPlus(dimensions, "<td>")
        i = string.find(dimensions, "<")
        rw.setDimensions(stripText(dimensions[0:i]))


    # Reading level
    i = string.find(source, ">Age Range:")

    if i != -1:
        readinglevel = searchForPlus(source, ">Age Range:")
        readinglevel = searchForPlus(readinglevel, "<td>")
        i = string.find(readinglevel, "<")
        rw.setReadingLevel(stripText(readinglevel[0:i]))


    # Lexile
    i = string.find(source, ">Lexile:")

    if i != -1:
        readinglevel = searchForPlus(source, ">Lexile:")
        readinglevel = searchForPlus(readinglevel, "<td>")
        i = string.find(readinglevel, "<")
        readinglevel = stripText(readinglevel[0:i])

        i = string.rfind(readinglevel, "(")

        if i != -1:
            readinglevel = stripText(readinglevel[0:i])

        rw.setLexileLevel(readinglevel)


    # Find List price
    i = string.find(source, "class=\"old-price\"")

    if i != -1:
        listprice = searchFor(source, "class=\"old-price\"")
        listprice = searchFor(listprice, "$")
        i = string.find(listprice, "<")
        rw.setListPrice(stripText(listprice[0:i]))


    # Find Retail Price (value)
    i = string.find(source, "itemprop=\"price\"")

    if i != -1:
        value = searchFor(source, "itemprop=\"price\"")
        value = searchFor(value, ">")
        i = string.find(value, "<")
        rw.setItemValue(value[0:i])

        rw.setPurchasePrice(rw.getItemValue())
        rw.setPurchaseDate(Date())
        rw.setPurchasePlace(rw.getSource())


    # Find Category
    i = string.rfind(source, ">Related Subjects<")

    if i != -1:
        categories = stripText(source[i:])
        i = string.find(categories, "</section>")
        categories = stripText(categories[0:i])

        i = string.find(categories, "href=\"")

        if i != -1:
            categories = searchForPlus(categories, "href=\"")
            categories = searchForPlus(categories, ">")
            i = string.find(categories, "<")
            category = stripText(categories[0:i])
            rw.setCategory1(category)

        i = string.find(categories, "href=\"")

        if i != -1:
            categories = searchForPlus(categories, "href=\"")
            categories = searchForPlus(categories, ">")
            i = string.find(categories, "<")
            category = stripText(categories[0:i])
            rw.setCategory2(category)

        i = string.find(categories, "href=\"")

        if i != -1:
            categories = searchForPlus(categories, "href=\"")
            categories = searchForPlus(categories, ">")
            i = string.find(categories, "<")
            category = stripText(categories[0:i])
            rw.setCategory3(category)


    # Find Comments
    comments = ""

    i = string.rfind(source, "class=\"book-seller-cntnt\"")

    if i != -1:
        comment = searchForPlus(source, "class=\"book-seller-cntnt\"")
        comment = searchForPlus(comment, "\">")
        i = string.find(comment, "</div>")
        comment = stripText(comment[0:i])

        comments = appendComment(comments, comment)

    i = string.rfind(source, "class=\"overview-cntnt\"")

    if i != -1:
        comment = searchForPlus(source, "class=\"overview-cntnt\"")
        comment = searchForPlus(comment, "\">")
        i = string.find(comment, "</div>")
        comment = stripText(comment[0:i])

        comments = appendComment(comments, comment)

    i = string.find(source, "class=\"review first\"")

    if i != -1:
        source = searchForPlus(source, "class=\"review first\"")
        review = searchForPlus(source, "<h4>")
        i = string.find(review, "<")
        review = stripText(review[0:i]) + "<br>"

        i = string.find(source, "class=\"full-review\"")

        if i == -1 or i > 1000:
            comment = searchForPlus(source, "</h4>")
            i = string.find(comment, "</div>")
        else:
            comment = searchForPlus(source, "class=\"full-review\"")
            comment = searchForPlus(comment, "\">")
            i = string.find(comment, "</div>")

        comment = stripText(comment[0:i])

        if comment.startswith("<P>") == 1:
            comment = stripText(comment[3:])

        if comment.startswith("<p>") == 1:
            comment = stripText(comment[3:])

        comment = review + comment
        comments = appendComment(comments, comment)

    i = string.find(source, "Editorial Reviews</h2>")

    if i != -1:
        reviews = searchForPlus(source, "Editorial Reviews</h2>")
        i = string.find(reviews, "</div>")
        reviews = stripText(reviews[0:i])
        comments = appendComment(comments, "<br>Editorial Reviews")

        while (searchForPlus(reviews, "<blockquote>") != None):
            reviews = searchForPlus(reviews, "<blockquote>")
            i = string.find(reviews, "</blockquote>")
            review = stripText(reviews[0:i])
            review = string.replace(review, "<footer>", "- ")
            review = string.replace(review, "</footer>", "<br>")
            comments = appendComment(comments, review)

    i = string.find(source, "<h5>From The Critics</h5>")

    if i != -1:
        reviews = searchFor(source, "<h5>From The Critics</h5>")
        i = string.find(reviews, "</section>")
        reviews = stripText(reviews[0:i])
        reviews = string.replace(reviews, "</h5>", "<br>")
        reviews = reviews + "<br>"
        comments = appendComment(comments, reviews)

    comments = string.replace(comments, "<ol>", "")
    comments = string.replace(comments, "</ol>", "")

    # Fixed in 
    comments = string.replace(comments, "\xE2\x80\x94", "--")
    comments = string.replace(comments, "&#8213;", "--")
    comments = string.replace(comments, "\xE2\x80\x98", "'")
    comments = string.replace(comments, "\xE2\x80\xA6", "...")
    comments = string.replace(comments, "\xE2\x98\x85", "*")
    comments = string.replace(comments, "&#9733; ", "*")
    comments = string.replace(comments, "&#9733;", "*")
    rw.setProductInfo(comments)


def extractTextbook(source):
    # Defaults
    rw.setPublicationPlace("US")
    rw.setLanguage("English")
    rw.setCurrencySymbol("$")


    # Used pricing
    i = string.find(source, "<em>Marketplace</em>")

    if i != -1:
        prices = searchForPlus(source, "<em>Marketplace</em>")
        prices = searchForPlus(prices, "class=\"price\"")
        prices = searchForPlus(prices, ">")
        i = string.find(prices, "<")
        rw.setUsedPrice(stripText(prices[0:i]));
        rw.setNewPrice(stripText(prices[0:i]));

        i = string.find(prices, "<em>Used</em>")

        if i != -1:
            counts = searchForPlus(prices, "<em>Used</em>")
            counts = searchForPlus(counts, "(")
            i = string.find(counts, ")")
            rw.setUsedCount(stripText(counts[0:i]));

        i = string.find(prices, ">New<")

        if i != -1:
            counts = searchForPlus(prices, ">New<")
            counts = searchForPlus(counts, "(")
            i = string.find(counts, ")")
            rw.setNewCount(stripText(counts[0:i]));

        


    # Main extraction
    # Find Image
    i = string.find(source, "data-bntrack=\"Product Image\"")

    if i != -1:
        images = searchForPlus(source, "data-bntrack=\"Product Image\"")
        image = searchForPlus(images, "src=\"")
        i = string.find(image, "\"")
        image = stripText(image[0:i])

        i = string.find(image, "nookbook_badge")

        if i != -1:
            image = searchForPlus(images, "src=\"")
            image = searchForPlus(image, "src=\"")
            i = string.find(image, "\"")
            image = stripText(image[0:i])

        i = string.find(image, "ImageNA")

        if i != -1:
            image = ""

        rw.setImage1(image)

    i = string.find(source, "data-bn-src-url=\"")

    if i != -1:
        images = searchForPlus(source, "data-bn-src-url=\"")
        i = string.find(images, "\"")
        image = stripText(images[0:i])
        rw.setLargeImage1(image)



    # Find Title
    source = searchForPlus(source, "class=\"w-box wgt-productTitle\"")
    source = searchForPlus(source, "<h1")
    source = searchForPlus(source, ">")
    i = string.find(source, "<")
    title = stripText(source[0:i])

    i = string.rfind(title, "(")

    if i != -1:
        title = stripText(title[0:i])

    if title.endswith("/") == 1:
        title = stripText(title[0:len(title) -1])

    rw.setTitle(title)


    # Find Author
    authorList = []
    i = string.find(source, "data-bntrack=\"Contributor_1\"")

    if i != -1:
        source = searchForPlus(source, "data-bntrack=\"Contributor_1\"")
        i = string.find(source, "</div>")
        authors = stripText(source[0:i])

        while (searchFor(authors, "\">") != None):
            authors = searchForPlus(authors, "\">")
            i = string.find(authors, "<")
            author = stripText(authors[0:i])
            authors = searchForPlus(authors, "</a>")
            authors = stripText(authors)

            if authors.startswith("(") == 1:
                i = string.find(authors, ")")
                role = stripText(authors[0:i+1])

                if role == "(Illustrator)":
                    rw.setIllustrator(author)
                    author = ""
                elif role == "(Translator)":
                    rw.setTranslator(author)
                    author = ""
                elif role == "(Editor)":
                    rw.setEditor(author)
                    author = ""
                else:
                    author = author + " " + role
                
            if author != "":
                authorList.append(author)

    rw.setAuthors(authorList)


    # ISBN
    i = string.find(source, ">ISBN-13:")

    if i != -1:
        isbn = searchForPlus(source, ">ISBN-13:")
        isbn = searchForPlus(isbn, ">")
        i = string.find(isbn, "<")
        rw.setISBN(stripText(isbn[0:i]))


    # Find Publisher
    i = string.find(source, ">Publisher:")

    if i != -1:
        publisher = searchForPlus(source, ">Publisher:")
        i = string.find(publisher, "<")
        rw.setPublisher(stripText(publisher[0:i]))


    # Find Publication date
    i = string.find(source, ">Pub. Date:")

    if i != -1:
        date = searchForPlus(source, ">Pub. Date:")
        i = string.find(date, "<")
        rw.setReleaseDate(stripText(date[0:i]))


    # Find Sales rank
    i = string.find(source, ">Sales Rank:")

    if i != -1:
        rank = searchForPlus(source, ">Sales Rank:")
        i = string.find(rank, "<")
        rw.setSalesRank(stripText(rank[0:i]))


    # Series
    i = string.find(source, ">Series:")

    if i != -1:
        series = searchForPlus(source, ">Series:")
        series = searchForPlus(series, "\">")
        i = string.find(series, "<")
        rw.setSeries(stripText(series[0:i]))


        # Series Number
        i = string.find(source, ", #")

        if i != -1:
            series = searchForPlus(source, ", #")
            i = string.find(series, "<")
            rw.setSeriesNumber(stripText(series[0:i]))


    # Find List price
    i = string.find(source, "class=\"list-price")

    if i != -1:
        listprice = searchFor(source, "class=\"list-price")
        listprice = searchFor(listprice, "$")
        i = string.find(listprice, "<")
        rw.setListPrice(stripText(listprice[0:i]))


    # Find Retail Price (value)
    i = string.find(source, "class=\"price\"")

    if i != -1:
        value = searchFor(source, "class=\"price\"")
        value = searchFor(value, "$")
        i = string.find(value, "<")
        rw.setItemValue(value[0:i])

        rw.setPurchasePrice(rw.getItemValue())
        rw.setPurchaseDate(Date())
        rw.setPurchasePlace(rw.getSource())


    # Find Comments
    comments = ""

    i = string.rfind(source, "<h3>Overview")

    if i != -1:
        comment = searchFor(source[i-20:], "<h3>Overview")
        i = string.find(comment, "<div id=")
        comment = stripText(comment[0:i])
        comments = appendComment(comments, comment)

        i = string.rfind(comments, "<h3>Product Details<")

        if i != -1:
            comments = searchForPlus(comments, "<h3>Product Details<")
            comments = searchForPlus(comments, "<h3>")



    i = string.find(source, "class=\"review first\"")

    if i != -1:
        source = searchForPlus(source, "class=\"review first\"")
        review = searchForPlus(source, "<h4>")
        i = string.find(review, "<")
        review = stripText(review[0:i]) + "<br>"

        i = string.find(source, "class=\"full-review\"")

        if i == -1 or i > 1000:
            comment = searchForPlus(source, "</h4>")
            i = string.find(comment, "</div>")
        else:
            comment = searchForPlus(source, "class=\"full-review\"")
            comment = searchForPlus(comment, "\">")
            i = string.find(comment, "</div>")

        comment = stripText(comment[0:i])

        if comment.startswith("<P>") == 1:
            comment = stripText(comment[3:])

        if comment.startswith("<p>") == 1:
            comment = stripText(comment[3:])

        comment = review + comment
        comments = appendComment(comments, comment)

    i = string.find(source, "<h3>Editorial Reviews</h3>")

    if i != -1:
        reviews = searchForPlus(source, "<h3>Editorial Reviews</h3>")
        i = string.find(reviews, "</section>")
        reviews = stripText(reviews[0:i])
        comments = appendComment(comments, "<br>Editorial Reviews")

        while (searchForPlus(reviews, "class=\"fade-to-height box") != None):
            reviews = searchForPlus(reviews, "class=\"fade-to-height box")
            review = searchForPlus(reviews, "<h5>")
            i = string.find(review, "</div>")
            review = stripText(review[0:i])
            review = string.replace(review, "</h5>", "<br>")
            review = review + "<br>"
            comments = appendComment(comments, review)

    i = string.find(source, "<h5>From The Critics</h5>")

    if i != -1:
        reviews = searchFor(source, "<h5>From The Critics</h5>")
        i = string.find(reviews, "</section>")
        reviews = stripText(reviews[0:i])
        reviews = string.replace(reviews, "</h5>", "<br>")
        reviews = reviews + "<br>"
        comments = appendComment(comments, reviews)

    # Fixed in 
    comments = string.replace(comments, "\xE2\x80\x94", "--")
    comments = string.replace(comments, "\xE2\x80\x98", "'")
    comments = string.replace(comments, "\xE2\x80\xA6", "...")
    comments = string.replace(comments, "\xE2\x98\x85", "*")
    rw.setProductInfo(comments)




def getImageURL(url):
    http = HTTPConnection()
    http.resetReferer();
    http.blockForLoad();
    url =  string.replace(url, "&amp;", "&")
    images = http.getContents(url)
    t2 = open(rw.getLogsDir() + "trace2.html", "w")
    t2.write(images)
    t2.close()

    i = string.rfind(images, "class=\"DisplayPane\"")

    if i != -1:
        images = stripText(images[i:])
        images = searchForPlus(images, "src=\"")
        i = string.find(images, "\"")
        return stripText(images[0:i])

        i = string.find(imagesline, "'")

        if i != -1:
            image = searchForPlus(imagesline, "'")
            i = string.find(image, "'")
            image = stripText(image[0:i])
            return image




try:
    rw.setBookObject(Book)

    i = string.find(source, ">Textbook Details<")

    if i == -1:
        extract(source)
    else:
        extractTextbook(source)

finally:
    if os.path.exists(rw.getDocsDir() + "rwuserexit.py"):
        execfile(rw.getDocsDir() + "rwuserexit.py") in globals()
