# This Python file uses the following encoding: iso-8859-1
# Library and Archives Canada scraper
#
# Copyright  1999-2009 Readerware Corporation.  All Rights Reserved.

import  os
import  string

import  rw

from    com.readerware.http import HTTPConnection

from    rwsupport import searchFor
from    rwsupport import searchForPlus
from    rwsupport import stripText
from    rwsupport import appendComment
from    rwsupport import calcISBN10CheckDigit

from    jarray import zeros, array
from    java.lang import String
from    java.util import Date


def extract(source):
    # Defaults
    rw.setDefaultPublicationPlace("Canada")
    rw.setDefaultLanguage("English")
    rw.setCurrencySymbol("$")


    i = string.find(source, "0 - 0 of 0")

    if i != -1:
        # Not found, try again with ISBN-10
        source = searchForPlus(source, "isbn:978")
        isbn = stripText(source[0:9])
        isbn = calcISBN10CheckDigit(isbn)
        url = "http://www.collectionscanada.gc.ca/lac-bac/results/lib?FormName=AMICUS+Simple+Search&PageNum=1&SortSpec=score+desc&Language=eng&QueryParser=lac_amicus&Sources=amicus&SearchIn_1=isbn&SearchInText_1=" + isbn + "&Operator_1=AND&SearchIn_2=&SearchInText_2=&Operator_2=AND&SearchIn_3=&SearchInText_3=&ParaUserView=&Publication=&SourceLanguage=+++&Format=&MaterialDateOperator=after&MaterialDate=&ResultCount=10&MaterialSource="
        http = HTTPConnection()
        http.resetReferer();
        http.blockForLoad();
        source = http.getContents(url)

        t2 = open(rw.getLogsDir() + "trace.html", "w")
        t2.write(source)
        t2.close()


    i = string.find(source, ">Library Search Result<")

    if i != -1:
        # Search results page, get detail page
        source = searchForPlus(source, ">Library Search Result<")
        source = searchForPlus(source, "href=\"")
        i = string.find(source, "\"")
        source = stripText(source[0:i])
        source = string.replace(source, "&amp;", "&")
        http = HTTPConnection()
        http.resetReferer();
        http.blockForLoad();
        source = http.getContents(source)

        t2 = open(rw.getLogsDir() + "trace.html", "w")
        t2.write(source)
        t2.close()


    # Main extraction
    # Find Author
    authorList = []
    i = string.find(source, "NAME(S):")

    if i != -1:
        source = searchFor(source, "NAME(S):")
        i = string.find(source, "TITLE(S):")
        authors = stripText(source[0:i])

        while (searchFor(authors, "HREF=\"") != None):
            authors = searchForPlus(authors, "HREF=\"")
            authors = searchForPlus(authors, "\">")
            i = string.find(authors, "<")
            author = stripText(authors[0:i])
            i = string.rfind(author, "-")

            if i != -1:
                i = string.rfind(author, ",")
                author = stripText(author[0:i])

            authorList.append(author)

        rw.setAuthors(authorList)


    # Find Title
    source = searchForPlus(source, "TITLE(S):")
    i = string.find(source, "\n")
    title = stripText(source[0:i])

    i = string.rfind(title, " / ")

    if i != -1:
        title = stripText(title[0:i])

    rw.setTitle(title)


    # Find Publisher
    i = string.find(source, "PUBLISHER:")

    if i != -1:
        place = ""
        copyDate = ""

        publisher = searchForPlus(source, "PUBLISHER:")
        i = string.find(publisher, "\n")
        publisher = stripText(publisher[0:i])

        i = string.find(publisher, " : ")

        if i != -1:
            place = stripText(publisher[0:i])
            publisher = stripText(publisher[i+2:])

        i = string.find(publisher, ", c")

        if i != -1:
            copyDate = stripText(publisher[i+3:])
            publisher = stripText(publisher[0:i])

            i = string.find(copyDate, ".")

            if i != -1:
                copyDate = stripText(copyDate[0:i])

        rw.setPublisher(publisher)
        rw.setPublicationPlace(place)
        rw.setCopyrightDate(copyDate)


    # Find Comments, part 1
    comments = ""
    i = string.find(source, "DESCRIPTION:")

    if i != -1:
        comment = searchForPlus(source, "DESCRIPTION:")
        i = string.find(comment, "S:")
        comment = stripText(comment[0:i])

        i = string.find(comment, "N:")

        if i != -1:
            comment = stripText(comment[0:i])

        i = string.rfind(comment, " ")

        if i != -1:
            comment = stripText(comment[0:i])

        comments = comment
        rw.setProductInfo(comments)

        i = string.find(comment, " p.")
        
        if i != -1:
            pages = stripText(comment[0:i])
            i = string.rfind(pages, " ")

            if i != -1:
                pages = stripText(pages[i:])

            rw.setPages(pages)

        i = string.find(comment, " cm.")
        
        if i != -1:
            dimensions = stripText(comment[0:i])
            i = string.rfind(dimensions, " ")

            if i != -1:
                dimensions = stripText(dimensions[i:])

            rw.setDimensions(dimensions + " cm.")

        i = string.find(comment, " in.")
        
        if i != -1:
            dimensions = stripText(comment[0:i])
            i = string.rfind(dimensions, " ")

            if i != -1:
                dimensions = stripText(dimensions[i:])

            rw.setDimensions(dimensions + " in.")


    # Find Series
    i = string.find(source, "SERIES:")

    if i != -1:
        series = searchForPlus(source, "SERIES:")
        series = searchForPlus(source, "\">")
        i = string.find(series, "<")
        series = stripText(series[0:i])
        rw.setSeries(series)


    # Find Comments, part 2
    i = string.find(source, "NOTES:")

    if i != -1:
        comment = searchForPlus(source, "NOTES:")
        i = string.find(comment, "S:")
        comment = stripText(comment[0:i])

        i = string.find(comment, "N:")

        if i != -1:
            comment = stripText(comment[0:i])

        i = string.rfind(comment, " ")

        if i != -1:
            comment = stripText(comment[0:i])
            
        comments = appendComment(comments, comment)
        rw.setProductInfo(comments)


    # Find LCCN
    i = string.find(source, "LCCN:")

    if i != -1:
        lccn = searchForPlus(source, "LCCN:")
        i = string.find(lccn, "\n")
        lccn = stripText(lccn[0:i])
        rw.setLCCN(lccn)


    # Find ISBN
    i = string.find(source, "ISBN:")

    if i != -1:
        isbn = searchForPlus(source, "ISBN:")
        i = string.find(isbn, "\n")
        isbn = stripText(isbn[0:i])

        i = string.find(isbn, " : ")

        if i != -1:
            value = stripText(isbn[i+2:])
            isbn = stripText(isbn[0:i])

        i = string.find(isbn, " (")

        if i != -1:
            isbn = stripText(isbn[0:i])

        rw.setISBN(isbn)


    # Find Call Number
    i = string.find(source, "LC Call no.:")

    if i != -1:
        callnumber = searchForPlus(source, "LC Call no.:")
        callnumber = searchForPlus(callnumber, "\">")
        i = string.find(callnumber, "<")
        callnumber = stripText(callnumber[0:i])
        rw.setCallNumber(callnumber)


    # Find Dewey Decimal Number
    i = string.find(source, "Dewey:")

    if i != -1:
        dewey = searchForPlus(source, "Dewey:")
        dewey = searchForPlus(dewey, "\">")
        i = string.find(dewey, "<")
        dewey = stripText(dewey[0:i])
        rw.setDewey(dewey)


    # Find Keywords
    i = string.find(source, "SUBJECTS:")

    if i != -1:
        keywords = ""
        temp = searchForPlus(source, "SUBJECTS:")
        i = string.find(temp, "</PRE>")
        temp = stripText(temp[0:i])

        while (searchFor(temp, "HREF=\"") != None):
            temp = searchForPlus(temp, "HREF=\"")
            temp = searchForPlus(temp, "\">")
            i = string.find(temp, "<")
            keyword = stripText(temp[0:i])

            if keywords == "":
                keywords = keyword
                category = keyword
            else:
                keywords = keywords + ", " + keyword

        rw.setKeywords(keywords)






try:
    rw.setBookObject(Book)
    extract(source)

finally:
    if os.path.exists(rw.getDocsDir() + "rwuserexit.py"):
        execfile(rw.getDocsDir() + "rwuserexit.py") in globals()

