# This Python file uses the following encoding: iso-8859-1
# chapters.ca scraper
#
# Copyright  1999-2020 Readerware Corporation.  All Rights Reserved.

import  os
import  string
import  time

import  rw

from    com.readerware.http import HTTPConnection

from    rwsupport import searchFor
from    rwsupport import searchForPlus
from    rwsupport import stripText
from    rwsupport import appendComment

from    jarray import zeros, array
from    java.lang import String
from    java.util import Date


def extract(source):
    # Defaults
    rw.setPublicationPlace("Canada")
    rw.setDefaultLanguage("English")
    cSymbol = "$"
    rw.setCurrencySymbol(cSymbol)


    # Main extraction

    # Find Image
    i = string.find(source, "itemProp=\"image\"")

    if i != -1:
        image = source[i:]
        image = searchForPlus(image, "itemProp=\"image\"")
        image = searchForPlus(image, "href=\"")
        i = string.find(image, "\"")
        image = stripText(image[0:i])

        while (searchFor(image, "&amp;") != None):
            i = string.find(image, "&amp;")
            image = image[0:i+1] + image[i+5:]

        rw.setImage1(image)
        rw.setLargeImage1(image)


    # Find Title
    source = searchForPlus(source, "id=\"product-section-details\"")
    source = searchForPlus(source, "\"item-page__main-title\"")
    source = searchForPlus(source, "\">")
    i = string.find(source, "<")
    title = stripText(source[0:i])
    rw.setTitle(title)


    # Find Authors
    tag = "\"contributor-link-Author-"
    i = string.find(source, tag)

    if i != -1:
        authors = source
        authorList = []

        while (searchFor(authors, tag) != None):
            authors = searchForPlus(authors, tag)
            authors = searchForPlus(authors, "\">")
            i = string.find(authors, "<")
            author = stripText(authors[0:i])
                
            if author != "":
                authorList.append(author)

        rw.setAuthors(authorList)


    # Find Price
    i = string.find(source, "class=\"item-price__price-amount\"")

    if i != -1:
        value = searchForPlus(source, "class=\"item-price__price-amount\"")
        value = searchFor(value, "$")
        i = string.find(value, "<")
        value = stripText(value[0:i])
        rw.setItemValue(value)
        rw.setPurchasePrice(rw.getItemValue())
        rw.setPurchaseDate(Date())
        rw.setPurchasePlace(rw.getSource())


    # Find List Price
    i = string.find(source, "class=\"item-price__list-price\"")

    if i != -1:
        value = searchForPlus(source, "class=\"item-price__list-price\"")
        value = searchFor(value, "$")
        i = string.find(value, "<")
        value = stripText(value[0:i])
        rw.setListPrice(value)


    # Find Details/Specs section
    i = string.find(source, "id=\"DetailsAndSpecsContainer\"")

    if i != -1:
        specs = searchForPlus(source, "id=\"DetailsAndSpecsContainer\"")

        # Find Format
        i = string.find(specs, "\"spec-value--Format\"")

        if i != -1:
            val = searchForPlus(specs, "\"spec-value--Format\"")
            val = searchForPlus(val, ">")
            i = string.find(val, "<")
            val = stripText(val[0:i])
            rw.setFormat(val)

        # Find Release date
        i = string.find(specs, "\"spec-value--ReleaseDate\"")

        if i != -1:
            val = searchForPlus(specs, "\"spec-value--ReleaseDate\"")
            val = searchForPlus(val, ">")
            i = string.find(val, "<")
            val = stripText(val[0:i])
            rw.setReleaseDate(val)

        # Find Dimensions/Pages
        i = string.find(specs, "\"spec-value--ProductDimensionsLabel\"")

        if i != -1:
            val = searchForPlus(specs, "\"spec-value--ProductDimensionsLabel\"")
            val = searchForPlus(val, ">")
            i = string.find(val, "<")
            val = stripText(val[0:i])

            i = string.find(val, "pages,")

            if i != -1:
                rw.setPages(stripText(val[0:i]))

            rw.setDimensions(stripText(val[i+6:]))

        # Find Publisher
        i = string.find(specs, "\"spec-value--PublishedBy\"")

        if i != -1:
            val = searchForPlus(specs, "\"spec-value--PublishedBy\"")
            val = searchForPlus(val, ">")
            i = string.find(val, "<")
            val = stripText(val[0:i])
            rw.setPublisher(val)

        # Find Language
        i = string.find(specs, "\"spec-value--Language\"")

        if i != -1:
            val = searchForPlus(specs, "\"spec-value--Language\"")
            val = searchForPlus(val, ">")
            i = string.find(val, "<")
            val = stripText(val[0:i])
            rw.setLanguage(val)

        # Find ISBN
        i = string.find(specs, "\"isbn-value--ISBN-13\"")

        if i != -1:
            val = searchForPlus(specs, "\"isbn-value--ISBN-13\"")
            val = searchForPlus(val, ">")
            i = string.find(val, "<")
            val = stripText(val[0:i])
            rw.setISBN(val)


    # Find Comments
    comments = ""
    i = string.find(source, "class=\"item-description__content\"")

    if i != -1:
        desc = searchForPlus(source, "class=\"item-description__content\"")
        desc = searchForPlus(desc, ">")
        i = string.find(desc, "</div>")
        desc = stripText(desc[0:i])
        rw.setProductInfo(desc)




try:
    rw.setBookObject(Book)
    extract(source)
finally:
    if os.path.exists(rw.getDocsDir() + "rwuserexit.py"):
        execfile(rw.getDocsDir() + "rwuserexit.py") in globals()
