# This Python file uses the following encoding: iso-8859-1
# Chirp Books (audiobooks) scraper
#
# Copyright  1999-2022 Readerware Corporation.  All Rights Reserved.

import  os
import  string
import  time

import  rw

from    com.readerware.http import HTTPConnection

from    rwsupport import searchFor
from    rwsupport import searchForPlus
from    rwsupport import stripText
from    rwsupport import appendComment
from    rwsupport import calcISBN10CheckDigit
from    rwsupport import convertAuthor

from    jarray import zeros, array
from    java.lang import String
from    java.util import Date


def extract(source):
    # Defaults
    rw.setDefaultPublicationPlace("US")
    rw.setDefaultLanguage("English")
    cSymbol = "$"
    rw.setCurrencySymbol(cSymbol)
    rw.setFormat("Audio Book")

    print "Drag & Drop extraction..."
    source = searchForPlus(source, "id='main-content'")

    # Find Image
    tag = ";coverUrl&quot;"
    i = string.find(source, tag)

    if i != -1:
        images = searchForPlus(source, tag)
        images = searchForPlus(images, "&quot;")
        i = string.find(images, "&quot;")
        image = stripText(images[0:i])
        image = string.replace(image, "\u0026", "&")
        i = string.find(image, "width=")

        if i != -1:
            image = stripText(image[0:i+6])
            rw.setImage1(image + "256")
            rw.setLargeImage1(image + "512")
        else:
            rw.setImage1(image)


    # Find Title
    tag = "displayTitle&quot;"
    title = searchForPlus(source, tag)
    title = searchForPlus(title, "&quot;")
    i = string.find(title, "&quot;")
    title = stripText(title[0:i])
    rw.setTitle(title)


    # Find Authors
    authorList = []
    tag = "name&quot;"
    i = string.find(source, tag)

    if i != -1:
        authors = searchFor(source, tag)
        i = string.find(authors, "]")
        authors = stripText(authors[0:i])

        while (searchFor(authors, "name&quot;") != None):
            authors = searchForPlus(authors, "name&quot;")
            authors = searchForPlus(authors, "&quot;")
            i = string.find(authors, "&quot;")
            author = stripText(authors[0:i])

            if author != "":
                authorList.append(author)

    tag = "narratorCredits&quot;"
    i = string.find(source, tag)

    if i != -1:
        authors = searchFor(source, tag)
        i = string.find(authors, "]")
        authors = stripText(authors[0:i])

        while (searchFor(authors, "name&quot;") != None):
            authors = searchForPlus(authors, "name&quot;")
            authors = searchForPlus(authors, "&quot;")
            i = string.find(authors, "&quot;")
            author = stripText(authors[0:i])
            author = convertAuthor(author)

            if author != "":
                authorList.append(author + ", (Narrator)")

    rw.setAuthors(authorList)


    # Find Publisher
    i = string.find(source, "publisher&quot;")

    if i != -1:
        publisher = searchForPlus(source, "publisher&quot;")
        publisher = searchForPlus(publisher, "&quot;")
        i = string.find(publisher, "&quot;")
        publisher = stripText(publisher[0:i])
        rw.setPublisher(publisher)


    # Find publication date
    i = string.find(source, "releaseDate&quot;")

    if i != -1:
        relDate = searchForPlus(source, "releaseDate&quot;")
        relDate = searchForPlus(relDate, "&quot;")
        i = string.find(relDate, "&quot;")
        relDate = stripText(relDate[0:i])
        rw.setReleaseDate(relDate)


    # Find Comments
    comments = ""

    tag = "editorialDescription&quot;"

    if string.find(source, tag) != -1:
        reviews = searchForPlus(source, tag)
        reviews = searchForPlus(reviews, "&quot;")
        i = string.find(reviews, "&quot;")
        comment = "Audiobook Overview<br />" + stripText(reviews[0:i])
        comments = appendComment(comments, comment)

    tag = "bookDescription&quot;"

    if string.find(source, tag) != -1:
        reviews = searchForPlus(source, tag)
        reviews = searchForPlus(reviews, "&quot;")
        i = string.find(reviews, "&quot;")
        comment = "Publisher Description<br />" + stripText(reviews[0:i])
        comments = appendComment(comments, comment)

    comments = string.replace(comments, "\u003ci\u003e", "")
    comments = string.replace(comments, "\u003c/i\u003e", "")
    comments = string.replace(comments, "\u003cp\u003e", "")
    comments = string.replace(comments, "\u003c/p\u003e", "")
    rw.setProductInfo(comments)


    # Find list price/value
    i = string.find(source, "listingPrice&quot;")

    if i != -1:
        listPrice = searchForPlus(source, "listingPrice&quot;")
        listPrice = searchForPlus(listPrice, "&quot;")
        i = string.find(listPrice, "&quot;")
        listPrice = stripText(listPrice[0:i])
        rw.setListPrice(listPrice)
        rw.setItemValue(listPrice)
        rw.setPurchasePrice(rw.getItemValue())
        rw.setPurchaseDate(Date())
        rw.setPurchasePlace(rw.getSource())


    # Find value
    i = string.find(source, "discountPrice&quot;")

    if i != -1:
        value = searchForPlus(source, "discountPrice&quot;")
        value = searchForPlus(value, "&quot;")
        i = string.find(value, "&quot;")
        value = stripText(value[0:i])
        rw.setItemValue(value)
        rw.setPurchasePrice(rw.getItemValue())
        rw.setPurchaseDate(Date())
        rw.setPurchasePlace(rw.getSource())


    # Find Keywords
    i = string.find(source, "tags&quot;")

    if i != -1:
        extracted = ""
        keywords = searchForPlus(source, "tags&quot;")
        i = string.find(keywords, "]")
        keywords = stripText(keywords[0:i])

        while (searchFor(keywords, "displayName&quot;") != None):
            keywords = searchForPlus(keywords, "displayName&quot;")
            keywords = searchForPlus(keywords, "&quot;")
            i = string.find(keywords, "&quot;")
            keyword = stripText(keywords[0:i])

            if extracted == "":
                extracted = keyword
            else:
                extracted = extracted + ", " + keyword

        rw.setKeywords(extracted)


    # Find running time
    i = string.find(source, "runtime&quot;")

    if i != -1:
        runTime = searchForPlus(source, "runtime&quot;")
        runTime = searchForPlus(runTime, "&quot;")
        i = string.find(runTime, "&quot;")
        runTime = stripText(runTime[0:i])
        rw.setDimensions(runTime)



try:
    rw.setBookObject(Book)
    extract(source)

finally:
    if os.path.exists(rw.getDocsDir() + "rwuserexit.py"):
        execfile(rw.getDocsDir() + "rwuserexit.py") in globals()
