# Internet Archive book scraper
#
# Copyright  1999-2022 Readerware Corporation.  All Rights Reserved.

import  os
import  string
import  time

import  rw

from    com.readerware.http import HTTPConnection

from    rwsupport import searchFor
from    rwsupport import searchForPlus
from    rwsupport import stripText
from    rwsupport import appendComment
from    rwsupport import calcISBN10CheckDigit

from    jarray import zeros, array
from    java.lang import String
from    java.util import Date


def extract(source):
    # Defaults
    rw.setDefaultPublicationPlace("US")
    rw.setDefaultLanguage("English")

    print "Looking for results page..."

    if string.find(source, "item-ttl") != -1:
        # results page, follow link
        print "Results page found!"
        url = searchForPlus(source, "item-ttl")
        url = searchForPlus(url, "href=\"")
        i = string.find(url, "\"")
        url = "https://archive.org" + stripText(url[0:i])

        print "Getting item from:",url

        http = HTTPConnection()
        http.blockForLoad();
        source = http.getContents(url, "UTF-8")
        
        try:
            t = open(rw.getLogsDir() + "trace2.html", "w")
            t.write(source)
            t.close()
        except:
            print "Unable to create trace2.html"


    # Find Image
    tag = "twitter:image"
    i = string.find(source, tag)

    if i == -1:
        tag = "og:image"
        i = string.find(source, tag)

    if i != -1:
        images = searchForPlus(source, tag)
        images = searchForPlus(images, "content=\"")
        i = string.find(images, "\"")
        image = stripText(images[0:i])
        rw.setLargeImage1(image)
        rw.setImage1(image)


    # Find Title
    tag = "class=\"item-title\""
    source = searchForPlus(source, tag)
    source = searchForPlus(source, "itemprop=\"name\"")
    source = searchForPlus(source, ">")
    i = string.find(source, "<")
    title = stripText(source[0:i])
    rw.setTitle(title)


    # Find Authors
    authorList = []
    tag = "<dt>by</dt>"
    i = string.find(source, tag)

    if i != -1:
        source = searchFor(source, tag)
        i = string.find(source, "</div>")
        authors = stripText(source[0:i])

        while (searchFor(authors, "href=\"") != None):
            authors = searchForPlus(authors, "href=\"")
            authors = searchForPlus(authors, "\">")
            i = string.find(authors, "<")
            author = stripText(authors[0:i])

            i = string.find(authors, ", 19")

            if i != -1:
                author = stripText(authors[0:i])

            i = string.find(authors, ", 18")

            if i != -1:
                author = stripText(authors[0:i])

            i = string.find(authors, ", 17")

            if i != -1:
                author = stripText(authors[0:i])

            if author.endswith(", author") == 1:
                author = stripText(author[0:len(author)-8])

            if author.endswith(", joint author") == 1:
                author = stripText(author[0:len(author)-14])

            if author.endswith(", ed") == 1:
                author = stripText(author[0:len(author)-4])
                rw.setEditor(author)
                author = ""
                
            if author != "":
                authorList.append(author)

    rw.setAuthors(authorList)


    # Find Publication Date
    i = string.find(source, ">Publication date<")

    if i != -1:
        date = searchForPlus(source, ">Publication date<")
        date = searchForPlus(date, "itemprop=\"datePublished\"")
        date = searchForPlus(date, ">")
        i = string.find(date, "<")
        date = stripText(date[0:i])
        rw.setReleaseDate(date)


    # Find Publisher
    i = string.find(source, ">Publisher<")

    if i != -1:
        publisher = searchForPlus(source, ">Publisher<")
        publisher = searchForPlus(publisher, "itemprop=\"publisher\"")
        publisher = searchForPlus(publisher, ">")
        i = string.find(publisher, "<")
        publisher = stripText(publisher[0:i])

        if publisher.startswith("[") == 1:
            publisher = stripText(publisher[1:])
            i = string.find(publisher, "]")
            place = stripText(publisher[0:i])
            rw.setPublicationPlace(place)
            publisher = stripText(publisher[i+1:])

            if publisher.startswith(":") == 1:
                publisher = stripText(publisher[1:])

        i = string.find(publisher, " : ")

        if i != -1:
            place = stripText(publisher[0:i])
            rw.setPublicationPlace(place)
            publisher = stripText(publisher[i+3:])

        rw.setPublisher(publisher)


    # Find Keywords
    i = string.find(source, ">Topics<")
    allKeywords = ""

    if i != -1:
        keywords = searchForPlus(source, ">Topics<")
        i = string.find(keywords, "</dl>")
        keywords = stripText(keywords[0:i])

        while (searchFor(keywords, "href=\"") != None):
            keywords = searchForPlus(keywords, "href=\"")
            keywords = searchForPlus(keywords, "\">")
            i = string.find(keywords, "<")
            keyword = stripText(keywords[0:i])

            if keyword.endswith(".") == 1:
                keyword = keyword[0:len(keyword)-1]

            if allKeywords != "":
                allKeywords = allKeywords + ", "

            allKeywords = allKeywords + keyword

        rw.setKeywords(allKeywords)


    # Find Language
    i = string.find(source, ">Language<")

    if i != -1:
        language = searchForPlus(source, ">Language<")
        language = searchForPlus(language, "href=\"")
        language = searchForPlus(language, "\">")
        i = string.find(language, "<")
        language = stripText(language[0:i])
        rw.setLanguage(language)


    # Find Book Info
    i = string.find(source, "id=\"descript\"")

    if i != -1:
        info = searchForPlus(source, "id=\"descript\"")
        info = searchForPlus(info, "\">")
        i = string.find(info, "</div>")
        info = stripText(info[0:i])
        rw.setProductInfo(info)


    # Find ISBN
    i = string.find(source, ">Isbn<")

    if i != -1:
        isbn = searchForPlus(source, ">Isbn<")
        isbn = searchForPlus(isbn, "itemprop=\"isbn\"")
        isbn = searchForPlus(isbn, ">")
        i = string.find(isbn, "<")
        isbn = stripText(isbn[0:i])
        rw.setISBN(isbn)


    # Find LCCN
    i = string.find(source, ">Lccn<")

    if i != -1:
        lccn = searchForPlus(source, ">Lccn<")
        lccn = searchForPlus(lccn, "class=\"\"")
        lccn = searchForPlus(lccn, ">")
        i = string.find(lccn, "<")
        lccn = stripText(lccn[0:i])
        rw.setLCCN(lccn)
    else:
        i = string.find(source, ">urn:lccn:")

        if i != -1:
            lccn = searchForPlus(source, ">urn:lccn:")
            i = string.find(lccn, "<")
            lccn = stripText(lccn[0:i])
            rw.setLCCN(lccn)


    # Find Page Count
    i = string.find(source, ">Pages<")

    if i != -1:
        pages = searchForPlus(source, ">Pages<")
        pages = searchForPlus(pages, "itemprop=\"numberOfPages\"")
        pages = searchForPlus(pages, ">")
        i = string.find(pages, "<")
        pages = stripText(pages[0:i])
        rw.setPages(pages)




try:
    rw.setBookObject(Book)
    extract(source)

finally:
    if os.path.exists(rw.getDocsDir() + "rwuserexit.py"):
        execfile(rw.getDocsDir() + "rwuserexit.py") in globals()
