# This Python file uses the following encoding: iso-8859-1
# Scraper for Koninklijke Bibliotheek [NL]
#   National library of the Netherlands
#   this is the nearest you will get to a reference library
#   of Dutch language works
#

import  os
import  string

import  rw

from    com.readerware.http import HTTPConnection

from    rwsupport import searchFor
from    rwsupport import searchForPlus
from    rwsupport import stripText
from    rwsupport import appendComment

from    jarray import zeros, array
from    java.lang import String
from    java.util import Date


def translate_text(str):
    str = string.replace(str, "\xC2\xA0", " ")
    str = string.replace(str, "\xC2\xAE", "")
    str = string.replace(str, "\xC3\xA7", "")
    str = string.replace(str, "\xC3\xA8", "")
    str = string.replace(str, "\xC3\xA9", "")
    str = string.replace(str, "\xC3\xAB", "")
    str = string.replace(str, "\xC3\xAD", "")
    str = string.replace(str, "\xC3\xAF", "")
    str = string.replace(str, "\xC5\xB8", "")
    str = string.replace(str, "\xC5\xBD", "")
    str = string.replace(str, "\xE2\x80\x93", "-")
    str = string.replace(str, "\xE2\x80\x98", "'")
    str = string.replace(str, "\xE2\x80\x99", "'")
    return string.strip(str);


# remove any <a> tags, keeping everything else
# assume PRE<a href="xxx">yyy</a>POST
# This returns PREyyyPOST
def removeAtags(str):

    cat = ""
    while (str != None):
        # save anything before
        tag = "<a"
        i = string.find(str, tag)
        if i == -1:
            tag = "<A"
            i = string.find(str, tag)
        if i == -1:
            cat = cat + str
            return cat
        else:
            if i > 0:
                cat = cat + str[:i]
                str = searchForPlus(str, tag)

        # move marker to "inside"
        tag = ">"
        i = string.find(str, tag)
        if i == -1:
            return cat
        else:
            str = searchForPlus(str, tag)

        tag = "</a>"
        i = string.find(str, tag)
        if i == -1:
            tag = "</A>"
            i = string.find(str, tag)
        if i == -1:
            return cat
        else:
            cat = cat + str[:i]
            str = searchForPlus(str, tag)


def translate_nl(str):
    str = string.replace (str, "Geschiedenis", "History")
    str = string.replace (str, "Pocket", "Mass Market Paperback")
    str = string.replace (str, "Populaire pockets", "Popular Reading")
    str = string.replace (str, "romans en novellen", "Literature")
    str = string.replace (str, "oorspr. - Nederlands", "Dutch")
    str = string.replace (str, "vertaald", "translated (Dutch)")
    str = string.replace (str, "geb.", "bound")
    str = string.replace (str, "pbk.", "paperback")
    str = string.replace (str, "woordenboeken", "Dictionaries")
    return string.strip(str);



def extract(source):
    # Defaults
    rw.setDefaultPublicationPlace("Netherlands")
    rw.setDefaultLanguage("Dutch")
    rw.setCurrencySymbol("[[EUR]]")

    # save base if it exists
    i = string.find (source, "BASE href=\"")
    if i != -1:
        base = searchForPlus(source,  "BASE href=\"");
        i = string.find (base, "\"")
        base = stripText(base[0:i])
    else:
        base = "" 

    # is this the right page?
    start_tag = ">Results<"
    j = string.find (source, start_tag)
    if j == -1: 
        ## print "Can't parse page"
        return
    else:
        source = searchForPlus(source, start_tag)

    # First case - no match
    i = string.find (source, "There are no hits.")
    if i != -1:
        # not found
        print "No hits"
        return

    # either we have the book page, or a list of editions sharing the
    # same ISBN

    # test for the multiple case.
    i = string.find (source, "There are")
    if i > 0:
        hits = searchForPlus(source, "There are")
        i = string.find (hits, "hits")
        hits = stripText(hits[0:i]);

        print "Hits:", hits
        i = string.find (source, "This is hit 1")
        if i == -1:
            # just use the first and hope that it is correct

            if base == "":
                print "no base page"
                return

            url = base + "LNG=EN/SHW?FRST=1"
            http = HTTPConnection()
            http.blockForLoad();
            source = http.getContents(url)
            t = open(rw.getLogsDir() + "trace2.html", "w")
            t.write(source)
            t.close()

            comments = "Extracted one edition out of choice from KB NL- check for correct edition"


    # If a list of possible ISBNs returned, search again for ISBN-13
    i = string.find (source, "&nbsp;search term&nbsp;")

    if i != -1:
        source = searchForPlus(source, "&nbsp;(isbn, issn)&nbsp;")
        source = searchForPlus(source, "\">")
        i = string.find (source, "<")
        isbn13 = stripText(source[0:i]);

        if len(isbn13) == 10:
            isbn13 = "978" + isbn13[0:9]
            isbn13 = calcISBN13CheckDigit(isbn13)

            url = "http://opc4.kb.nl/DB=1/SET=1/LNG=EN/CMD?ACT=SRCHA&IKT=1007&SRT=RLV&TRM=" + isbn13
            http = HTTPConnection()
            http.resetReferer();
            http.blockForLoad();
            source = http.getContents(url)
            t = open(rw.getLogsDir() + "trace2.html", "w")
            t.write(source)
            t.close()


    #
    # this should be the product page
    #

    # Find Title
    source = searchForPlus(source, ">Title:")
    source = searchForPlus(source, "<div>")
    i = string.find(source, "</div>")
    title = stripText(source[0:i])

    i = string.find(title, " / ")

    if i != -1:
        title = stripText(title[0:i])

    rw.setTitle(title)

    # Find Author
    i = string.find(source, ">Author:")

    if i != -1:
        author = searchForPlus(source, ">Author:")
        author = searchForPlus(author, "href")
        author = searchForPlus(author, "\">")
        i = string.find(author, "<")
        author = stripText(author[0:i])

        i = string.find(author, "(")

        if i != -1:
            author = stripText(author[0:i])

        rw.setAuthor(author)



    # Find Year
    i = string.find(source, ">Year:")

    if i != -1:
        year = searchForPlus(source, ">Year:")
        year = searchForPlus(year, "<span>")
        i = string.find(year, "<")
        year = stripText(year[0:i])

        if year.startswith("cop.") == 1:
            year = searchForPlus(year, "cop.")

        rw.setCopyrightDate(year)


    # Find Publisher + Place
    i = string.find(source, ">Publisher:")

    if i != -1:
        publisher = searchForPlus(source, ">Publisher:")
        place = searchForPlus(source, ">Publisher:")
        place = searchForPlus(place, "<span>")
        i = string.find(place, "<")
        place = stripText(place[0:i])

        i = string.find(place, "[")

        if i != -1:
            place = stripText(place[0:i])

        rw.setPublicationPlace(place)

        publisher = searchForPlus(publisher, "<span>")
        publisher = searchForPlus(publisher, "<span>")
        i = string.find(publisher, "</div>")
        publisher = stripText(publisher[0:i])
        rw.setPublisher(publisher)


    # Find Comments
    i = string.find(source, ">Note:")

    if i != -1:
        info = searchForPlus(source, ">Note:")
        info = searchForPlus(info, "<span>")
        i = string.find(info, "<")
        info = stripText(info[0:i])
        info = translate_text(info)
        rw.setProductInfo(info)


    # Find Pages
    i = string.find(source, ">Extent:")

    if i != -1:
        pages = searchForPlus(source, ">Extent:")
        pages = searchForPlus(pages, "<span>")
        i = string.find(pages, "<")
        pages = stripText(pages[0:i])

        i = string.find(pages, " ")

        if i != -1:
            pages = stripText(pages[0:i])

        rw.setPages(pages)


    # Find Dimensions
    i = string.find(source, ">Size:")

    if i != -1:
        dimensions = searchForPlus(source, ">Size:")
        dimensions = searchForPlus(dimensions, "<span>")
        i = string.find(dimensions, "<")
        dimensions = stripText(dimensions[0:i])
        rw.setDimensions(dimensions)


    # Find ISBN
    i = string.find(source, ">ISBN:")

    if i != -1:
        isbn = searchForPlus(source, ">ISBN:")
        isbn = searchForPlus(isbn, "<span>")
        i = string.find(isbn, "<")
        isbn = stripText(isbn[0:i])
        rw.setISBN(isbn)








try:
    rw.setBookObject(Book)
    extract(source)
finally:
    if os.path.exists(rw.getDocsDir() + "rwuserexit.py"):
        execfile(rw.getDocsDir() + "rwuserexit.py") in globals()
