# This Python file uses the following encoding: iso-8859-1
# The National Bibliography of Finland scraper
#
# Copyright  1999-2013 Readerware Corporation.  All Rights Reserved.

import  os
import  string

import  rw

from    com.readerware.http import HTTPConnection

from    rwsupport import searchFor
from    rwsupport import searchForPlus
from    rwsupport import stripText
from    rwsupport import appendComment
from    rwsupport import calcISBN10CheckDigit

from    jarray import zeros, array
from    java.lang import String
from    java.util import Date


def extract(source):
    # Defaults
    rw.setDefaultPublicationPlace("Finland")


    i = string.find(source, ">Hakutuloksia ei l&ouml;ytynyt.<")
        
    if i != -1:
        # Not found, try again with ISBN-10
        source = searchForPlus(source, ">Hakutuloksia ei l&ouml;ytynyt.<")
        i = string.find(source, ">Hakusana(t):")

        if i != -1:
            source = searchForPlus(source, ">Hakusana(t):")
            source = searchForPlus(source, "value=\"")
            i = string.find(source, "\"")
            isbn = stripText(source[0:i])

            if isbn.startswith("978"):
                isbn = stripText(isbn[3:])
                isbn = stripText(isbn[0:9])
                isbn = calcISBN10CheckDigit(isbn)
        
                http = HTTPConnection()
                http.resetReferer();
                http.blockForLoad();
                source = http.getContents("https://fennica.linneanet.fi/vwebv/search?searchArg=" + isbn + "+&searchCode=020B&limitTo=none&recCount=10&searchType=1&page.search.search.button=Search")

                t2 = open(rw.getLogsDir() + "trace2.html", "w")
                t2.write(source)
                t2.close()

    i = string.find(source, ">Hakutuloksia ei l&ouml;ytynyt.<")
        
    if i != -1:
        # Not found
        return

    # Main extraction
    # Find Author
    authorList = []
    i = string.find(source, ">Tekij&auml;:<")

    if i != -1:
        source = searchFor(source, ">Tekij&auml;:<")
        i = string.find(source, "</li>")
        authors = stripText(source[0:i])

        while (searchFor(authors, "href=\"") != None):
            authors = searchForPlus(authors, "href=\"")
            authors = searchForPlus(authors, "\">")
            i = string.find(authors, "<")
            author = stripText(authors[0:i])

            authorList.append(author)

    rw.setAuthors(authorList)


    # Find Title
    title = searchForPlus(source, ">Nimeke:<")
    title = searchForPlus(title, "\">")
    i = string.find(title, "<")
    title = stripText(title[0:i])
    title = cleanupData(title)

    i = string.rfind(title, " / ")

    if i != -1:
        title = stripText(title[0:i])

    rw.setTitle(title)


    # Find Title
    title = searchForPlus(source, ">Teos:<")
    title = searchForPlus(title, "href=\"")
    title = searchForPlus(title, "\">")
    i = string.find(title, "<")
    title = stripText(title[0:i])
    title = cleanupData(title)

    i = string.rfind(title, " / ")

    if i != -1:
        title = stripText(title[0:i])

    rw.setSubtitle(title)


    # Find Publisher
    i = string.find(source, ">Julkaistu:<")

    if i != -1:
        date = ""
        copyDate = ""
        publisher = searchForPlus(source, ">Julkaistu:<")
        publisher = searchForPlus(publisher, "\">")
        i = string.find(publisher, "<")
        publisher = stripText(publisher[0:i])
        publisher = cleanupData(publisher)

        i = string.find(publisher, " : ")

        if i != -1:
            place = stripText(publisher[0:i])
            place = string.replace(place, "[", "")
            place = string.replace(place, "]", "")
            publisher = stripText(publisher[i+2:])
            rw.setPublicationPlace(place)

        i = string.find(publisher, ", ")

        if i != -1:
            date = stripText(publisher[i+1:])
            publisher = stripText(publisher[0:i])

            if date.startswith("cop.") == 1:
                copyDate = stripText(date[4:])
                date = "";

        rw.setPublisher(publisher)

        i = string.find(date, " ")

        if i != -1:
            date = stripText(date[0:i])

        rw.setReleaseDate(date)

        i = string.find(copyDate, " ")

        if i != -1:
            copyDate = stripText(copyDate[0:i])

        rw.setCopyrightDate(copyDate)

            


    # Find Pages/Dimensions
    i = string.find(source, ">Ulkoasu:<")

    if i != -1:
        attr = searchForPlus(source, ">Ulkoasu:<")
        attr = searchForPlus(attr, "\">")
        i = string.find(attr, "<")
        attr = stripText(attr[0:i])

        i = string.find(attr, " s.")

        if i != -1:
            pages = stripText(attr[0:i])

            i = string.find(pages, ",")

            if i != -1:
                pages = stripText(attr[0:i])

            rw.setPages(pages)

        i = string.find(attr, " ; ")

        if i != -1:
            attr = stripText(attr[i+2:])

        i = string.find(attr, " cm")

        if i != -1:
            dimensions = stripText(attr[0:i+3])
            rw.setDimensions(dimensions)


    # Series
    i = string.find(source, ">Series:<")

    if i != -1:
        series = searchForPlus(source, ">Series:<")
        series = searchForPlus(series, "<TD")
        series = searchForPlus(series, ">")
        i = string.find(series, "<")
        series = stripText(series[0:i])
        series = cleanupData(series)


    # Find ISBN
    i = string.find(source, ">ISBN:<")

    if i != -1:
        isbn = searchForPlus(source, ">ISBN:<")
        isbn = searchForPlus(isbn, "\">")
        i = string.find(isbn, "<")
        isbn = stripText(isbn[0:i])

        i = string.find(isbn, "(nid.)")

        if i != -1:
            rw.setFormat("Paperback")

        i = string.find(isbn, "(hft.)")

        if i != -1:
            rw.setFormat("Paperback")

        i = string.find(isbn, "(sid.)")

        if i != -1:
            rw.setFormat("Hardcover")

        i = string.find(isbn, "(inb.)")

        if i != -1:
            rw.setFormat("Hardcover")

        i = string.find(isbn, "(")

        if i != -1:
            isbn = stripText(isbn[0:i])

        rw.setISBN(isbn)


    # Find Keywords
    i = string.find(source, ">Subject:<")

    if i != -1:
        keywords = ""
        temp = searchForPlus(source, ">Subject:<")
        i = string.find(temp, "<TH")
        temp = stripText(temp[0:i])

        while (searchFor(temp, "HREF=\"") != None):
            temp = searchForPlus(temp, "HREF=\"")
            temp = searchForPlus(temp, "\">")
            i = string.find(temp, "<")
            keyword = stripText(temp[0:i])
            keyword = cleanupData(keyword)

            i = string.find(keyword, "(")

            if i != -1:
                keyword = stripText(keyword[0:i])

            if keywords == "":
                keywords = keyword
                category = keyword
            else:
                keywords = keywords + ", " + keyword



def cleanupData(str):
    str = string.replace(str, "\xC3\x84", "\xC4")
    str = string.replace(str, "\xC3\x85", "\xC5")
    str = string.replace(str, "\xC3\x96", "\xD6")
    str = string.replace(str, "\xC3\xA4", "\xE4")
    str = string.replace(str, "\xC3\xA5", "\xE5")
    str = string.replace(str, "\xC3\xB6", "\xF6")
    str = string.replace(str, "\xC5\x8D", "\xF6")
    return str    




try:
    rw.setBookObject(Book)
    extract(source)
finally:
    if os.path.exists(rw.getDocsDir() + "rwuserexit.py"):
        execfile(rw.getDocsDir() + "rwuserexit.py") in globals()
