# This Python file uses the following encoding: iso-8859-1
# Library of Congress scraper (OLD SITE as of May 2014, catalog2.loc.gov)
#
# Copyright  1999-2007 Readerware Corporation.  All Rights Reserved.

import  os
import  string

import  rw

from    com.readerware.http import HTTPConnection

from    rwsupport import searchFor
from    rwsupport import searchForPlus
from    rwsupport import stripText
from    rwsupport import appendComment

from    jarray import zeros, array
from    java.lang import String
from    java.util import Date


def extract(source):
    # Defaults
    rw.setDefaultPublicationPlace("US")
    rw.setDefaultLanguage("English")
    rw.setCurrencySymbol("$")


    # Looking for the ISBN I searched for. LOC
    # can return multiple ISBNs for the same listing
    # want to get the correct format.
    searchISBN = ""
    i = string.find(source, "Guided Search = (")

    if i != -1:
        searchISBN = searchForPlus(source, "Guided Search = (")
        i = string.find(searchISBN, ")")
        searchISBN = stripText(searchISBN[0:i])

        if len(searchISBN) < 10:
            searchISBN = ""
        else:
            # ISBN 10 excl check digit
            searchISBN = stripText(searchISBN[3:i])
            searchISBN = stripText(searchISBN[0:len(searchISBN)-1])


    # Looking for duplicate LCCN matches, take one with LCCN only, no prefix.
    while (string.find(source, ">[&nbsp;") != -1):
        i = string.find(source, ">[&nbsp;")

        if (i != -1):
            source = stripText(source[i-500:])
            source = searchForPlus(source, "<A HREF=\"")
            i = string.find(source, "\"")
            url = "http://catalog.loc.gov" + stripText(source[0:i])
            http = HTTPConnection()
            http.resetReferer();
            http.blockForLoad();
            newsource = http.getContents(url)
            i = string.find(newsource, ">LC control no.:<")
            lccndata = newsource[i:i+100]
            i = string.find(lccndata, "   <STRONG>")

            if i != -1:
                source = newsource
                break

            source = searchForPlus(source, ">[&nbsp;")
            t2 = open(rw.getLogsDir() + "trace2.html", "w")
            t2.write(source)
            t2.close()
            

    # Determine page format
    i = string.find(source, "view1a.gif")

    if i != -1:
        source = searchForPlus(source, "view2.gif")
        source = searchForPlus(source, "<A HREF=\"")
        i = string.find(source, "\"")
        url = "http://catalog.loc.gov" + stripText(source[0:i])
        http = HTTPConnection()
        http.resetReferer();
        http.blockForLoad();
        source = http.getContents(url)
        t3 = open(rw.getLogsDir() + "trace3.html", "w")
        t3.write(source)
        t3.close()


    # Find Title
    title = searchForPlus(source, "view3a.gif")

    if title == None:
        return

    title = searchForPlus(title, "<STRONG>")
    title = searchForPlus(title, "\">")
    i = string.find(title, "<")
    title = stripText(title[0:i])
    i = string.find(title, " / ")

    if i != -1:
        title = stripText(title[0:i])

    # Remove Translated by, if present.
    i = string.find(title, ". Translated")

    if (i != -1):
        title = title[0:i]

    # Remove trailing period, if present
    if title.endswith(".") == 1:
        title = title[0:len(title)-1]

    rw.setTitle(title)


    # Find LCCN
    i = string.find(source, ">LC control no.:<")

    if i != -1:
        lccn = searchForPlus(source, ">LC control no.:<")
        lccn = searchForPlus(lccn, "<TD")
        lccn = searchForPlus(lccn, ">")
        i = string.find(lccn, "</TD>")
        lccn = stripText(lccn[0:i])
        rw.setLCCN(lccn)


    # Find Author (it is possible to have no author line, in
    # which case I will assume "Various", i.e. a collection with no
    # specific author
    authorList = []
    i = string.find(source, ">Personal name:<")

    if i != -1:
        authors = searchForPlus(source, ">Personal name:<")
        authors = searchForPlus(authors, "<TD")
        authors = searchForPlus(authors, ">")
        i = string.find(authors, "</TD>")
        authors = stripText(authors[0:i])
        author = searchForPlus(authors, "HREF=")
        author = searchForPlus(author, ">")
        i = string.find(author, "<")
        author = stripText(author[0:i])

        # Remove , pseud if present.
        if (string.count(author, ",") > 1):
            i = string.find(author, ",")
            i = string.find(author, ",", i+1)
            author = author[0:i]

        # Remove trailing period, if present
        i = string.rfind(author, ".")

        if (i != -1):
            x = i - 2

            if (x >= 2):
                if (author[x:x+1] != ' '):
                    if (len(author) == (i + 1)):
                        author = author[0:i]
            else:
                if (len(author) == (i + 1)):
                    author = author[0:i]

        if author[len(author) - 1] == ",":
            author = author[0: len(author)-1]

        authorList.append(rw.convertName(author))


    i = string.find(source, ">Related Names:<")

    if i != -1:
        authors = searchForPlus(source, ">Related Names:<")
        authors = searchForPlus(authors, "<TD")
        authors = searchForPlus(authors, ">")
        i = string.find(authors, "</TD>")
        author = stripText(authors[0:i])
        author = searchForPlus(author, "HREF=")
        author = searchForPlus(author, ">")
        i = string.find(author, "<")
        author = stripText(author[0:i])

        # We do need to get related names as sometimes this is the only
        # author information. But I also get some strange data at times
        # (978-0-553-56526-3). If it doesn't look like an author, skip
        i = string.find(author, ",")

        if i != -1:
            # Looks like last, first
            # Remove , pseud if present.
            if (string.count(author, ",") > 1):
                i = string.find(author, ",")
                i = string.find(author, ",", i+1)
                author = author[0:i]

            # Remove trailing period, if present
            i = string.rfind(author, ".")

            if (i != -1):
                x = i - 2

                if (x >= 2):
                    if (author[x:x+1] != ' '):
                        if (len(author) == (i + 1)):
                            author = author[0:i]
                else:
                    if (len(author) == (i + 1)):
                        author = author[0:i]

            if author[len(author) - 1] == ",":
                author = author[0: len(author)-1]

            authorList.append(rw.convertName(author))

    rw.setAuthors(authorList)


    # Find Place
    i = string.find(source, ">Published/")

    if i != -1:
        attrs = searchForPlus(source, ">Published/")
        attrs = searchForPlus(attrs, "<TD")
        attrs = searchForPlus(attrs, ">")
        attrs = stripText(attrs)
        i = string.find(attrs, "</TD>")
        attrs = stripText(attrs[0:i])
        i = string.find(attrs, " : ")

        if i != -1:
            rw.setPublicationPlace(stripText(attrs[0:i]))
            attrs = stripText(attrs[i+2:])
        else:
            i = string.find(attrs, "., ")

            if i != -1:
                rw.setPublicationPlace(stripText(attrs[0:i+1]))
                attrs = stripText(attrs[i+2:])
            else:
                i = string.find(attrs, ", ")

                if i != -1:
                    rw.setPublicationPlace(stripText(attrs[0:i]))
                    attrs = stripText(attrs[i+1:])


        # Find Publisher
        i = string.rfind(attrs, ",")

        if i != -1:
            rw.setPublisher(stripText(attrs[0:i]))
            attrs = stripText(attrs[i+1:])
        else:
            i = string.rfind(attrs, "[")

            if i != -1:
                rw.setPublisher(stripText(attrs[0:i]))
                attrs = stripText(attrs[i+1:])
            else:
                rw.setPublisher(attrs)


        # Find Publication Date
        i = string.find(attrs, ".")

        if i != -1:
            date = stripText(attrs[0:i])
        else:
            date = attrs

        i = string.find(date, "[")

        if i != -1:
            date = stripText(date[i+1:])

        i = string.find(date, "]")

        if i != -1:
            date = stripText(date[0:i])

        # if date in format cyyyy, strip c
        if (len(date) == 5):
            i = string.find(date, "c")

            if i == 0:
                date = stripText(date[1:])
                rw.setCopyrightDate(date)
        
        rw.setReleaseDate(date)


    # Find Pages
    i = string.find(source, ">Description:<")

    if i != -1:
        attrs = searchForPlus(source, ">Description:<")
        attrs = searchForPlus(attrs, "<TD")
        attrs = searchForPlus(attrs, ">")
        i = string.find(attrs, "</TD>")
        attrs = stripText(attrs[0:i])
        i = string.find(attrs, "p.")

        if i != -1:
            pages = stripText(attrs[0:i])

            i = string.rfind(pages, " ")

            if i != -1:
                pages = stripText(pages[i:])

            rw.setPages(pages)

        i = string.find(attrs, "pages")

        if i != -1:
            pages = stripText(attrs[0:i])

            i = string.rfind(pages, " ")

            if i != -1:
                pages = stripText(pages[i:])

            rw.setPages(pages)


        # Find Dimensions
        dimensions = attrs
        i = string.find(dimensions, " cm.")

        if i != -1:
            dimensions = stripText(dimensions[i-5: i+4:])

        i = string.find(dimensions, " ")

        if i != -1:
            dimensions = stripText(dimensions[i:])

        i = string.find(dimensions, ";")

        if i != -1:
            dimensions = stripText(dimensions[i+1:])

        if dimensions == "cm.":
            dimensions = ""

        rw.setDimensions(dimensions)


    # Find ISBN
    i = string.find(source, ">ISBN:<")

    if i != -1:
        isbn = searchForPlus(source, ">ISBN:<")

        if searchISBN != "":
            i = string.find(isbn, searchISBN)

            if i != -1:
                if i > 30:
                    isbn = stripText(isbn[i-30:])
            
        isbn = searchForPlus(isbn, "<TD ")
        isbn = searchForPlus(isbn, ">")
        i = string.find(isbn, "</TD>")
        isbn = stripText(isbn[0:i])

        i = string.find(isbn, "<STRONG><I>")

        if i != -1:
            isbn = stripText(isbn[i+11:])

        # Find format (only look for paperback)
        i = string.find(isbn, "pbk")

        if (i != -1):
            rw.setFormat("Paperback")
        else:
            rw.setFormat("Hardcover")

        i = string.find(isbn, "<")

        if i != -1:
            isbn = stripText(isbn[0:i])

        i = string.find(isbn, " ")

        if i != -1:
            isbn = stripText(isbn[0:i])

        rw.setISBN(isbn)


    # Find Comments
    comments = ""
    i = string.find(source, ">Notes:<")

    if i != -1:
        comment = searchForPlus(source, ">Notes:<")
        comment = searchForPlus(comment, "<TD")
        comment = searchForPlus(comment, ">")
        i = string.find(comment, "</TD>")
        comment = stripText(comment[0:i])
        comments = appendComment(comments, comment)


    # Find Comments
    i = string.find(source, ">Summary:<")

    if i != -1:
        comment = searchForPlus(source, ">Summary:<")
        comment = searchForPlus(comment, "<TD")
        comment = searchForPlus(comment, ">")
        i = string.find(comment, "</TD>")
        comment = stripText(comment[0:i])
        comments = appendComment(comments, comment)

    if comments != "":
        rw.setProductInfo(comments)


    # Find Keywords
    i = string.find(source, ">Subjects:<")

    if i != -1:
        temp = searchForPlus(source, ">Subjects:<")
        i = string.find(temp, "<TH")
        temp = stripText(temp[0:i])
        keywords = ""

        while (searchFor(temp, "HREF") != None):
            temp = searchForPlus(temp, "HREF")
            temp = searchForPlus(temp, "\">")
            i = string.find(temp, "<")
            keyword = stripText(temp[0:i])
            i = string.rfind(keyword, ".")

            if (i != -1):
                keyword = stripText(keyword[0:i])

            if keywords != "":
                keywords = keywords + ", " + keyword
            else:
                keywords = keyword

        rw.setKeywords(keywords)


    # Find Category
    i = string.find(source, ">Form/Genre:<")

    if (i != -1):
        source = searchForPlus(source, ">Form/Genre:<")
        source = searchForPlus(source, "<TD")
        source = searchForPlus(source, ">")
        source = searchForPlus(source, "HREF=")
        source = searchForPlus(source, ">")
        i = string.find(source, "<")
        category = stripText(source[0:i])

        i = string.rfind(category, ".")

        if i != -1:
            category = stripText(category[0:i])
            
        rw.setCategory1(category)


    # Find Series
    i = string.find(source, ">Series:<")

    if i != -1:
        source = searchForPlus(source, ">Series:<")
        source = searchForPlus(source, "\">")
        i = string.find(source, "<")
        rw.setSeries(stripText(source[0:i]))


    # Find LOC Call Number
    i = string.find(source, "LC classification:")

    if i != -1:
        source = searchForPlus(source, "LC classification:")
        source = searchForPlus(source, "\">")
        i = string.find(source, "<")
        rw.setCallNumber(stripText(source[0:i]))


    # Find Dewey Number
    i = string.find(source, ">Dewey class no.:<")

    if i != -1:
        source = searchForPlus(source, ">Dewey class no.:<")
        source = searchForPlus(source, "<TD")
        source = searchForPlus(source, ">")
        i = string.find(source, "</TD>")
        rw.setDewey(stripText(source[0:i]))







try:
    rw.setBookObject(Book)
    extract(source)
finally:
    if os.path.exists(rw.getDocsDir() + "rwuserexit.py"):
        execfile(rw.getDocsDir() + "rwuserexit.py") in globals()
